using UnityEngine;


[ExecuteInEditMode]
public class RandomObject : MonoBehaviour
{
    public enum vectorEnum { X, Y, Z, XY, XZ, YZ };
    [Header("Generation")]
    [Tooltip("The transform plane the randomization is based on")]
    public vectorEnum randomizeationVector;
    [Tooltip("The possible prefabs to randomize between")]
    public GameObject[] Objects; 
    [Header("Manual Control")]
    [Tooltip("Manually control the object")]
    public bool OverrideObject = false;    
    [Tooltip("Object index to use for manual override")]
    public int OverrideIndex = 0;
    [Header("Has Random Material")]
    [Tooltip("Calls the Randomize Material script if present on the objects.")]
    public bool UseRandomMaterial = false;
    [Header("Apply")]
    [Tooltip("This will apply all of the settings and delete the script")]
    public bool ApplyChanges = false;

    private GameObject objTaget = null;
    private GameObject targetObject = null;
    private GameObject currentprefab = null;
    private float p = 0f;
    private float f = 0.5f; // vector offset used to prevent a multiply by zero occurance. 
    private bool isPlay = false;

    void Update()
    {
        if (Application.isPlaying) // fail safe incase he script is still applied at run time.
        {
            isPlay = true;
            Destroy(this);
        }

        if (Objects.Length >= 1 && Objects[0] != null && !isPlay )  // this stop the script from removing the default object until the array has 1 object added.
        {           
            switch (randomizeationVector)
            {                
                case vectorEnum.X:
                    p = randomValueFromFloat(this.gameObject.transform.position.x);
                    break;
                case vectorEnum.Y:
                    p = randomValueFromFloat(this.gameObject.transform.position.y);
                    break;
                case vectorEnum.Z:
                    p = randomValueFromFloat(this.gameObject.transform.position.z);
                    break;
                case vectorEnum.XY:
                    p = randomValueFromFloat((this.gameObject.transform.position.x + f)*( this.gameObject.transform.position.y + f));
                    break;
                case vectorEnum.XZ:
                    p = randomValueFromFloat((this.gameObject.transform.position.x + f) * (this.gameObject.transform.position.y + f));
                    break;
                case vectorEnum.YZ:
                    p = randomValueFromFloat((this.gameObject.transform.position.y + f) * (this.gameObject.transform.position.z + f ));
                    break;
            }

            for (int x = 0; x < Objects.Length; x++)
            {
                if (applyObject(p, Objects.Length, x))
                {
                    targetObject = Objects[x];
                    break;
                }
            }

            if (OverrideObject)
            {
                if (OverrideIndex >= Objects.Length)
                {
                    OverrideIndex = Objects.Length - 1; // stops the index value going above the array limit.
                }
                targetObject = Objects[OverrideIndex];
            }
            if (Application.isEditor) // this is used to handle duplication of the child object when using play in editor.
            {
                foreach (Transform child in this.transform)
                {
                    GameObject.DestroyImmediate(child.gameObject);
                }
            }
            if (currentprefab != null)
            {
                DestroyImmediate(currentprefab);
            }
            currentprefab = Instantiate(targetObject); // creates the random object 
            currentprefab.transform.position = this.transform.position; // sets the random objects position to match source
            currentprefab.transform.parent = this.transform; //  attachs the random object to the parent
            if (UseRandomMaterial)
            {
                if (currentprefab.GetComponent<RandomMaterial>()) // checks to see if the current prefab has the Random Material script, if so updates that.
                {
                    RandomMaterial childScript = currentprefab.GetComponent<RandomMaterial>() as RandomMaterial;
                    childScript.Update();
                }                
            }
            if (ApplyChanges)
            {
                currentprefab.transform.parent = null;
                DestroyImmediate(this);
            }
        }
	}

    private float randomValueFromFloat(float seed)
    {
        Random.seed = (int)seed;
        return (Random.value);        
    }
    

    private bool applyObject(float p, int length, int x)
    {
        if (p < (1f / length) + ((float)x / length)){return (true);}
        else { return (false); }
    }

}